
#include "Extensions.h"

#ifndef OS_X
#ifdef X11
//whoops ;)
#define wglGetProcAddress(x) (*glXGetProcAddress)((const GLubyte*)(x))
#endif

// ARB_vertex_program + ARB_fragment_program
PFNGLGENPROGRAMSARBPROC					glGenProgramsARB					= 0;
PFNGLBINDPROGRAMARBPROC					glBindProgramARB					= 0;
PFNGLPROGRAMSTRINGARBPROC				glProgramStringARB					= 0;
PFNGLDELETEPROGRAMSARBPROC				glDeleteProgramsARB					= 0;
PFNGLPROGRAMENVPARAMETER4FARBPROC		glProgramEnvParameter4fARB			= 0;
PFNGLPROGRAMENVPARAMETER4FVARBPROC		glProgramEnvParameter4fvARB			= 0;
PFNGLPROGRAMLOCALPARAMETER4FARBPROC		glProgramLocalParameter4fARB		= 0;
PFNGLPROGRAMLOCALPARAMETER4FVARBPROC	glProgramLocalParameter4fvARB		= 0;

// ARB_vertex_buffer_object
PFNGLBINDBUFFERARBPROC					glBindBufferARB						= 0;
PFNGLDELETEBUFFERSARBPROC				glDeleteBuffersARB					= 0;
PFNGLGENBUFFERSARBPROC					glGenBuffersARB						= 0;
PFNGLBUFFERDATAARBPROC					glBufferDataARB						= 0;

// vertex array
//PFNGLDRAWRANGEELEMENTS				glDrawRangeElements					= 0;
#endif

// ripped from opengl.org :)
bool isExtensionSupported( const char *extension )
{
	const GLubyte		*extensions = NULL;
	const GLubyte		*start;
	GLubyte				*where, *terminator;

	/* Extension names should not have spaces. */
	where = ( GLubyte* )strchr( extension, ' ' );
	if( where || *extension == '\0' )
		return false;

	extensions = glGetString( GL_EXTENSIONS );

	/* It takes a bit of care to be fool-proof about parsing the
	OpenGL extensions string. Don't be fooled by sub-strings,
	etc. */
	start = extensions;

	for (;;) {
		where = ( GLubyte* )strstr( ( const char* ) start, extension );
		if( !where )
			break;

		terminator = where + strlen( extension );

		if( where == start || *( where - 1 ) == ' ' )
		if( *terminator == ' ' || *terminator == '\0' )
			return true;

		start = terminator;
	}

	return false;
}

bool loadExtensions() {

#ifndef OS_X
	if( isExtensionSupported( "GL_ARB_vertex_program" ) && isExtensionSupported( "GL_ARB_fragment_program" ) ) {

		glGenProgramsARB				= reinterpret_cast<PFNGLGENPROGRAMSARBPROC>( wglGetProcAddress( "glGenProgramsARB" ) );
		glBindProgramARB				= reinterpret_cast<PFNGLBINDPROGRAMARBPROC>( wglGetProcAddress( "glBindProgramARB" ) );
		glProgramStringARB				= reinterpret_cast<PFNGLPROGRAMSTRINGARBPROC>( wglGetProcAddress( "glProgramStringARB" ) );
		glDeleteProgramsARB				= reinterpret_cast<PFNGLDELETEPROGRAMSARBPROC>( wglGetProcAddress( "glDeleteProgramsARB" ) );
		glProgramLocalParameter4fARB	= reinterpret_cast<PFNGLPROGRAMLOCALPARAMETER4FARBPROC>( wglGetProcAddress( "glProgramLocalParameter4fARB" ) );
		glProgramLocalParameter4fvARB	= reinterpret_cast<PFNGLPROGRAMLOCALPARAMETER4FVARBPROC>( wglGetProcAddress( "glProgramLocalParameter4fvARB" ) );
		glProgramEnvParameter4fARB		= reinterpret_cast<PFNGLPROGRAMENVPARAMETER4FARBPROC>( wglGetProcAddress( "glProgramEnvParameter4fARB" ) );
		glProgramEnvParameter4fvARB		= reinterpret_cast<PFNGLPROGRAMENVPARAMETER4FVARBPROC>( wglGetProcAddress( "glProgramEnvParameter4fvARB" ) );

	} else {

		//MessageBox( 0, "vertex and/or fragment shader extensions not supported", "she-male detected!", MB_OK );
		return false;
	}

	if( isExtensionSupported( "GL_ARB_vertex_buffer_object" ) ) {

		glGenBuffersARB					= reinterpret_cast<PFNGLGENBUFFERSARBPROC>( wglGetProcAddress( "glGenBuffersARB" ) );
		glBindBufferARB					= reinterpret_cast<PFNGLBINDBUFFERARBPROC>( wglGetProcAddress( "glBindBufferARB" ) );
		glBufferDataARB					= reinterpret_cast<PFNGLBUFFERDATAARBPROC>( wglGetProcAddress( "glBufferDataARB" ) );
		glDeleteBuffersARB				= reinterpret_cast<PFNGLDELETEBUFFERSARBPROC>( wglGetProcAddress( "glDeleteBuffersARB" ) );

	} else {

		//MessageBox( 0, "vertex buffer extension not supported", "she-male detected!", MB_OK );
		return false;
	}

/*	if( isExtensionSupported( "GL_EXT_draw_range_elements" ) ) {

		glDrawRangeElements			= reinterpret_cast<PFNGLDRAWRANGEELEMENTS>( wglGetProcAddress( "glDrawRangeElements" ) );

	} else {

		MessageBox( 0, "vertex array range extension not supported", "she-male detected!", MB_OK );
		return false;
	 }*/
#endif
	return true;
} 
